/*
 * Copyright (C) 2000-2004 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 */
 
 /**
 * @file cc/cc1000.h
 *
 * @brief Chipcon CC1000 driver
 * 
 * 2004/06/19 Matthias Ringwald <mringwal@inf.ethz.ch>
 *
 */

#ifndef _CC_CC1000_H_
#define _CC_CC1000_H_

/** stores all information to tune cc1000 to a specific channel */
struct cc_channel {
    // 3 byte freq register A
    u_char  freqA[3];
    // 3 byte freq register B
    u_char  freqB[3];
    // 2 byte fsep
    u_char  fsep[2];
    // 1 byte rfdiv in pll
    u_char  refdiv;
    // 2 x 1 byte Charge_Pump override
    u_char  chp_co_rx;
    u_char  chp_co_tx;
    // 2 x 1 byte VCO_ARRAY override
    u_char  vco_ao_rx;
    u_char  vco_ao_tx;
    // 2 x 4 bytes: desired and actual frequency
    u_long  base;
    u_long  actFreq;
};

// Assistant functions for delaying.
// delay in the microsecond level
u_char delay_time(int u_sec);	

// Configuration APIs

// init the radio with specific frequency
void cc1000_init(void);

/* Set the radio to a specific frequency, given in hz */
void cc1000_set_frequency(u_long newFreqHz);

void cc1000_calibrate(void);
void cc1000_config(u_char count, short configuration[]);
void cc1000_write(u_char addr, u_char data);
u_char cc1000_read(u_char addr);

void cc1000_rxmode(void);
void cc1000_txmode(void);

int  cc1000_is_sleeping(void);
void cc1000_sleep(void);
void cc1000_wakeup(void);
void cc1000_on(void);

void cc1000_SPI_init(void);

// set und get radio transmission power
void cc1000_set_RF_power( u_char rfp );
void cc1000_get_RF_power( u_char* rfp );


/*
 * cc1000_calc_channel(u_long desiredFreq);
 *
 * Compute an achievable frequency and the necessary CC1000 parameters from
 * a given desired frequency (Hz) and stores them into a channel struct
 *
 * This routine assumes the following:
 *  - IF: 150 KHz                    (cc1000 constant)
 *  - Crystal Freq FOSC: 14.7456 MHz (BTnode3 & mica2 hardware config)
 *  - LO Injection: low              (could be changed/made configurable trivially)
 *  - Separation FSEP: 64000 Hz      (could be changed/made configurable trivialyl)
 *  - desired Frequency > 500 Mhz    (this assumption could be removed)
 *  
 * Note: "FOSC / 32768 = 450" helps a lot in reducing the terms :)
 * 
 */
void cc1000_calc_channel(u_long desiredFreq, struct cc_channel *chan);

/**
 * Set channel registers
 *
 * @param channel obtained by cc1000_calc_channel
 * @post  cc1000 registers are set. mode is not set
 */
void cc1000_set_channel(struct cc_channel *currChannel);
    

#endif /* _CC_CC1000_H_ */
